@@ -19,12 +19,12 @@ class DataFile:
     @staticmethod
     def default(environment: Optional[str]) -> "DataFile":
         """Creates new instance of `DataFile` initialized with default values"""
-        data_file = DataFile(environment, Settings(), {}, CustomDataInfo(None))
+        data_file = DataFile(environment, None, Settings(), {}, CustomDataInfo(None))
         return data_file
 
     @staticmethod
     def from_json(
-        environment: Optional[str], configuration: Dict[str, Any]
+        environment: Optional[str], last_modified: Optional[str], configuration: Dict[str, Any]
     ) -> "DataFile":
         """Creates new instance of `DataFile` initialized from the specified configuration JSON"""
         settings = Settings(configuration.get("configuration"))
@@ -34,23 +34,26 @@ class DataFile:
         }
         custom_data_info = CustomDataInfo(configuration.get("customData"))
         holdout = Experiment.from_json(configuration["holdout"]) if "holdout" in configuration else None
-        data_file = DataFile(environment, settings, feature_flags, custom_data_info, holdout)
+        data_file = DataFile(environment, last_modified, settings, feature_flags, custom_data_info, holdout)
         return data_file
 
     # pylint: disable=R0913
     def __init__(
         self,
         environment: Optional[str],
+        last_modified: Optional[str],
         settings: Settings,
         feature_flags: Dict[str, FeatureFlag],
         custom_data_info: CustomDataInfo,
         holdout: Optional[Experiment] = None,
     ) -> None:
         KameleoonLogger.debug(
-            'CALL: DataFile(environment: %s, settings: %s, feature_flags: %s, custom_data_info: %s, holdout: %s)',
-            environment, settings, feature_flags, custom_data_info, holdout
+            "CALL: DataFile(environment: %s, last_modified: %s, settings: %s, feature_flags: %s,"
+            " custom_data_info: %s, holdout: %s)",
+            environment, last_modified, settings, feature_flags, custom_data_info, holdout
         )
         self.__environment = environment  # pylint: disable=W0238
+        self.__last_modified = last_modified
         self.__settings = settings
         self.__feature_flags: Dict[str, FeatureFlag] = feature_flags
         self.__me_groups = self.__make_me_groups(feature_flags.values())
@@ -69,10 +72,16 @@ class DataFile:
         self.__custom_data_info = custom_data_info
         self.__holdout = holdout
         KameleoonLogger.debug(
-            'RETURN: DataFile.new(environment: %s, settings: %s, feature_flags: %s, custom_data_info: %s, holdout: %s)',
-            environment, settings, feature_flags, custom_data_info
+            "RETURN: DataFile(environment: %s, last_modified: %s, settings: %s, feature_flags: %s,"
+            " custom_data_info: %s, holdout: %s)",
+            environment, last_modified, settings, feature_flags, custom_data_info, holdout
         )
 
+    @property
+    def last_modified(self) -> Optional[str]:
+        """Returns last modified string"""
+        return self.__last_modified
+
     @property
     def settings(self) -> Settings:
         """Returns settings"""
@@ -177,11 +186,12 @@ class DataFile:
 
     def __str__(self):
         return (
-            f"DataFile{{"
+            "DataFile{"
             f"environment:'{self.__environment}',"
+            f"last_modified:'{self.__last_modified}',"
             f"feature_flags:{len(self.__feature_flags)},"
             f"settings:{self.__settings}"
-            f"}}"
+            "}"
         )
 
     class RuleInfo:
