@@ -45,9 +45,6 @@ class CustomData(Data, DuplicationUnsafeSendableBase):
     def data_type(self) -> DataType:
         return DataType.CUSTOM_DATA
 
-    def encode_query(self) -> str:
-        return super().encode_query() if len(self.__values) > 0 else ""
-
     def _add_query_params(self, query_builder: QueryBuilder) -> None:
         str_values = json.dumps({v: 1 for v in self.__values}, separators=(",", ":"))
         query_builder.extend(
@@ -58,7 +55,4 @@ class CustomData(Data, DuplicationUnsafeSendableBase):
         )
 
     def __str__(self):
-        return (
-            f"CustomData{{id:{self.__id},"
-            f"values:{self.__values}}}"
-        )
+        return f"CustomData{{id:{self.__id},values:{self.__values}}}"
