@@ -1,4 +1,6 @@
 """Visitor"""
+import time
+
 from threading import Lock
 from typing import Any, Callable, Optional, Dict, List, Iterator, Iterable, cast
 
@@ -15,6 +17,7 @@ from kameleoon.data.kcs_heat import KcsHeat
 from kameleoon.data.mapping_identifier import MappingIdentifier
 from kameleoon.data.operating_system import OperatingSystem
 from kameleoon.data.page_view import PageView
+from kameleoon.data.personalization import Personalization
 from kameleoon.data.unique_identifier import UniqueIdentifier
 from kameleoon.data.user_agent import UserAgent
 from kameleoon.data.manager.assigned_variation import AssignedVariation
@@ -43,6 +46,11 @@ class Visitor:  # pylint: disable=R0904
         """Creates and returns a clone of the current Visitor instance which shares all the visitor data."""
         return Visitor(self)
 
+    @property
+    def time_started(self) -> int:
+        """Returns the current visit's start time"""
+        return self._data.time_started
+
     def enumerate_sendable_data(self) -> Iterator[Sendable]:
         """Enumerates all sendable data associated with this Visitor."""
         KameleoonLogger.debug('CALL: Visitor.enumerate_sendable_data()')
@@ -127,6 +135,13 @@ class Visitor:  # pylint: disable=R0904
         KameleoonLogger.debug('CALL/RETURN: Visitor.variations -> (variations: %s)', variations)
         return variations
 
+    @property
+    def personalizations(self) -> Dict[int, Personalization]:
+        """Returns the personalizations associated with this Visitor."""
+        personalizations = EMPTY_DICT if self._data.personalizations is None else self._data.personalizations.copy()
+        KameleoonLogger.debug('CALL/RETURN: Visitor.personalizations -> (personalizations: %s)', personalizations)
+        return personalizations
+
     def get_forced_feature_variation(self, feature_key: str) -> Optional[ForcedFeatureVariation]:
         """
         Returns an associated forced feature variation by its feature key if it is present.
@@ -255,6 +270,7 @@ class Visitor:  # pylint: disable=R0904
     class VisitorData:
         """Visitor Data"""
         def __init__(self) -> None:
+            self.time_started = int(time.time() * 1000)
             self.lock = Lock()
             self.user_agent: Optional[str] = None
             self.device: Optional[Device] = None
@@ -266,6 +282,7 @@ class Visitor:  # pylint: disable=R0904
             self.page_view_visits: Optional[Dict[str, PageViewVisit]] = None
             self.conversions: Optional[List[Conversion]] = None
             self.variations: Optional[Dict[int, AssignedVariation]] = None
+            self.personalizations: Optional[Dict[int, Personalization]] = None
             self.forced_variations: Optional[Dict[int, ForcedExperimentVariation]] = None
             self.simulated_variations: Optional[Dict[str, ForcedFeatureVariation]] = None
             self.kcs_heat: Optional[KcsHeat] = None
@@ -280,18 +297,20 @@ class Visitor:  # pylint: disable=R0904
                 yield self.device
             if self.browser:
                 yield self.browser
-            if self.geolocation:
-                yield self.geolocation
             if self.operating_system:
                 yield self.operating_system
+            if self.geolocation:
+                yield self.geolocation
+            if self.visitor_visits:
+                yield self.visitor_visits
             if self.custom_data_dict is not None:
                 yield from list(self.custom_data_dict.values())
             if self.page_view_visits is not None:
                 yield from (visit.page_view for visit in list(self.page_view_visits.values()))
-            if self.conversions is not None:
-                yield from list(self.conversions)
             if self.variations is not None:
                 yield from list(self.variations.values())
+            if self.conversions is not None:
+                yield from list(self.conversions)
 
         def count_sendable_data(self) -> int:
             """Counts the total number of sendable data items associated with the visitor."""
@@ -300,18 +319,20 @@ class Visitor:  # pylint: disable=R0904
                 count += 1
             if self.browser is not None:
                 count += 1
+            if self.operating_system is not None:
+                count += 1
             if self.geolocation is not None:
                 count += 1
-            if self.operating_system is not None:
+            if self.visitor_visits:
                 count += 1
             if self.custom_data_dict is not None:
                 count += len(self.custom_data_dict)
             if self.page_view_visits is not None:
                 count += len(self.page_view_visits)
-            if self.conversions is not None:
-                count += len(self.conversions)
             if self.variations is not None:
                 count += len(self.variations)
+            if self.conversions is not None:
+                count += len(self.conversions)
             return count
 
     class DataAddingContext:
@@ -343,9 +364,16 @@ class Visitor:  # pylint: disable=R0904
         variation = cast(AssignedVariation, context.data)
         if self._data.variations is None:
             self._data.variations = {}
-        if context.overwrite or variation.experiment_id not in self.variations:
+        if context.overwrite or variation.experiment_id not in self._data.variations:
             self._data.variations[variation.experiment_id] = variation
 
+    def _add_personalization(self, context: DataAddingContext) -> None:
+        personalization = cast(Personalization, context.data)
+        if self._data.personalizations is None:
+            self._data.personalizations = {}
+        if context.overwrite or personalization.id not in self._data.personalizations:
+            self._data.personalizations[personalization.id] = personalization
+
     def _add_forced_feature_variation(self, context: DataAddingContext) -> None:
         ffv = cast(ForcedFeatureVariation, context.data)
         if self._data.simulated_variations is None:
@@ -417,7 +445,8 @@ class Visitor:  # pylint: disable=R0904
             self._data.cbscores = cast(CBScores, context.data)
 
     def _set_visitor_visits(self, context: DataAddingContext) -> None:
-        self._data.visitor_visits = cast(VisitorVisits, context.data)
+        if context.overwrite or (self._data.visitor_visits is None):
+            self._data.visitor_visits = cast(VisitorVisits, context.data).localize(self._data.time_started)
 
     def _set_is_unique_identifier(self, context: DataAddingContext) -> None:
         self._is_unique_identifier = cast(UniqueIdentifier, context.data).value
@@ -441,6 +470,7 @@ class Visitor:  # pylint: disable=R0904
         CBScores: _set_cbscores,
         VisitorVisits: _set_visitor_visits,
         AssignedVariation: _add_variation,
+        Personalization: _add_personalization,
         ForcedFeatureVariation: _add_forced_feature_variation,
         ForcedExperimentVariation: _add_forced_experiment_variation,
         UniqueIdentifier: _set_is_unique_identifier,
