@@ -47,7 +47,7 @@ from kameleoon.network.net_provider_impl import NetProviderImpl
 from kameleoon.network.network_manager_factory import NetworkManagerFactory
 from kameleoon.network.network_manager_factory_impl import NetworkManagerFactoryImpl
 from kameleoon.network.sendable import Sendable
-from kameleoon.network.services.configuration_service import ConfigurationService
+from kameleoon.network.services.configuration_service import ConfigurationService, FetchedConfiguration
 from kameleoon.network.cookie.cookie_manager import CookieManager
 
 from kameleoon.real_time.real_time_configuration_service import RealTimeConfigurationService
@@ -142,6 +142,7 @@ class KameleoonClient:
             config.default_timeout_second,
             NetProviderImpl(),
             atsf,
+            config.network_domain,
         )
         self._targeting_manager = TargetingManager(self._data_manager, self._visitor_manager)
         self._warehouse_manager = WarehouseManager(self._network_manager, self._visitor_manager)
@@ -354,7 +355,8 @@ class KameleoonClient:
         KameleoonLogger.info("RETURN: KameleoonClient.add_data(visitor_code: %s, args: %s)", visitor_code, args)
 
     def track_conversion(
-        self, visitor_code: str, goal_id: int, revenue: float = 0.0, is_unique_identifier: Optional[bool] = None
+        self, visitor_code: str, goal_id: int, revenue: float = 0.0, is_unique_identifier: Optional[bool] = None,
+        negative=False, metadata: Optional[Iterable[CustomData]] = None,
     ) -> None:
         """
         To track conversion, use the track_conversion() method. This method requires visitor_code and goal_id to track
@@ -372,28 +374,25 @@ class KameleoonClient:
         :param is_unique_identifier: (Deprecated) Parameter that specifies whether the visitorCode is
         a unique identifier.
         :type is_unique_identifier: Optional[bool]
-        :return: None
+        :param negative: Defines if the revenue is positive or negative. This field is optional (`False` by default).
+        :type negative: bool
+        :param metadata: Metadata of the conversion. This field is optional (`None` by default).
+        :type metadata: Optional[Iterable[CustomData]]
         """
         KameleoonLogger.info(
             "CALL: KameleoonClient.track_conversion(visitor_code: %s, goal_id: %s, revenue: %s, "
-            "is_unique_identifier: %s)",
-            visitor_code,
-            goal_id,
-            revenue,
-            is_unique_identifier,
+            "is_unique_identifier: %s, negative: %s, metadata: %s)",
+            visitor_code, goal_id, revenue, is_unique_identifier, negative, metadata
         )
         validate_visitor_code(visitor_code)
         if is_unique_identifier is not None:
             self.__set_unique_identifier(visitor_code, is_unique_identifier)
-        self.add_data(visitor_code, Conversion(goal_id, revenue))
+        self.add_data(visitor_code, Conversion(goal_id, revenue, negative, metadata))
         self._tracking_manager.add_visitor_code(visitor_code)
         KameleoonLogger.info(
             "RETURN: KameleoonClient.track_conversion(visitor_code: %s, goal_id: %s, revenue: %s, "
-            "is_unique_identifier: %s)",
-            visitor_code,
-            goal_id,
-            revenue,
-            is_unique_identifier,
+            "is_unique_identifier: %s, negative: %s, metadata: %s)",
+            visitor_code, goal_id, revenue, is_unique_identifier, negative, metadata
         )
 
     def flush(
@@ -1436,13 +1435,18 @@ class KameleoonClient:
         KameleoonLogger.debug("CALL: KameleoonClient._fetch_configuration(time_stamp: %s)", time_stamp)
         success = False
         try:
-            configuration_json = await self._obtain_configuration(time_stamp)
-            if configuration_json:
-                data_file = DataFile.from_json(self._config.environment, configuration_json)
-                self._data_manager.data_file = data_file
-                self._network_manager.url_provider.apply_data_api_domain(data_file.settings.data_api_domain)
-                self._call_update_handler_if_needed(time_stamp is not None)
-            success = True
+            fetched_configuration = await self._obtain_configuration(time_stamp)
+            if fetched_configuration:
+                if fetched_configuration.configuration:
+                    data_file = DataFile.from_json(
+                        self._config.environment,
+                        fetched_configuration.last_modified,
+                        fetched_configuration.configuration
+                    )
+                    self._data_manager.data_file = data_file
+                    self._network_manager.url_provider.apply_data_api_domain(data_file.settings.data_api_domain)
+                    self._call_update_handler_if_needed(time_stamp is not None)
+                success = True
         except Exception as ex:
             KameleoonLogger.error("Error occurred during configuration fetching: %s", ex)
         self._manage_configuration_update(self._data_manager.data_file.settings.real_time_update)
@@ -1533,7 +1537,7 @@ class KameleoonClient:
         if not self._data_manager.data_file.settings.real_time_update:
             await self._fetch_configuration()
 
-    async def _obtain_configuration(self, time_stamp: Optional[int]) -> Optional[Dict[str, Any]]:
+    async def _obtain_configuration(self, time_stamp: Optional[int]) -> Optional[FetchedConfiguration]:
         """
         Obtaining configuration from CDN service.
         Should be run in a separate thead.
@@ -1542,15 +1546,14 @@ class KameleoonClient:
         :return: None
         """
         KameleoonLogger.debug("CALL: KameleoonClient._obtain_configuration(time_stamp: %s)", time_stamp)
+        last_modified = self._data_manager.data_file.last_modified
         service: ConfigurationService = self._network_manager.get_service(ConfigurationService)
-        response = await service.fetch_configuration(self._config.environment, time_stamp)
-        content = None
-        if response.code and (response.code // 100 == 2):
-            content = response.content
+        fetched_configuration = await service.fetch_configuration(self._config.environment, time_stamp, last_modified)
         KameleoonLogger.debug(
-            "RETURN: KameleoonClient._obtain_configuration(time_stamp: %s) -> (content: %s)", time_stamp, content
+            "RETURN: KameleoonClient._obtain_configuration(time_stamp: %s) -> (fetched_configuration: %s)",
+            time_stamp, fetched_configuration
         )
-        return content
+        return fetched_configuration
 
     def __get_feature_variation_key(self, visitor_code: str, feature_key: str) -> Tuple[FeatureFlag, str]:
         """
