@@ -12,6 +12,7 @@ from kameleoon.configuration.rule_type import RuleType
 from kameleoon.data.visitor_visits import VisitorVisits
 from kameleoon.data.kcs_heat import KcsHeat
 from kameleoon.data.cbscores import CBScores, ScoredVarId
+from kameleoon.data.personalization import Personalization
 from kameleoon.data.manager.assigned_variation import AssignedVariation
 from kameleoon.data.manager.page_view_visit import PageViewVisit
 from kameleoon.data import (
@@ -27,12 +28,14 @@ from kameleoon.data import (
     OperatingSystemType,
     Data,
 )
+from kameleoon.types.remote_visitor_data_filter import RemoteVisitorDataFilter
 
 
 class RemoteVisitorData:
     """Remote visitor data"""
 
-    def __init__(self, json: Dict[str, Any]) -> None:
+    def __init__(self, json: Dict[str, Any], data_filter: RemoteVisitorDataFilter) -> None:
+        self.__filter = data_filter
         self.__geolocation: Optional[Geolocation] = None
         self.__device: Optional[Device] = None
         self.__browser: Optional[Browser] = None
@@ -40,19 +43,24 @@ class RemoteVisitorData:
         self.__page_view_visits: Optional[Dict[str, PageViewVisit]] = None
         self.__custom_data_dict: Optional[Dict[int, CustomData]] = None
         self.__experiments: Optional[Dict[int, AssignedVariation]] = None
+        self.__personalizations: Optional[Dict[int, Personalization]] = None
         self.__conversions: Optional[List[Conversion]] = None
-        self.__previous_visitor_visits: Optional[VisitorVisits] = None
+        self.__visit_number = 0
+        self.__visitor_visits: Optional[VisitorVisits] = None
         self.__visitor_code: Optional[str] = None
         current_visit = json.get("currentVisit")
         if current_visit is not None:
-            self.__parse_visit(current_visit)
+            self.__parse_visit(current_visit, False)
         previous_visits = json.get("previousVisits")
         if previous_visits:
-            times_started = []
+            prev_visits = []
             for visit in previous_visits:
-                times_started.append(visit.get("timeStarted", 0))
-                self.__parse_visit(visit)
-            self.__previous_visitor_visits = VisitorVisits(times_started)
+                time_started = visit.get("timeStarted", 0)
+                time_last_event = visit.get("timeLastEvent", time_started)
+                prev_visits.append(VisitorVisits.Visit(time_started, time_last_event))
+                self.__parse_visit(visit, True)
+            if len(prev_visits) > 0:
+                self.__visitor_visits = VisitorVisits(prev_visits, self.__visit_number)
         self.__kcs_heat = self.__parse_kcs_heat(json.get("kcs"))
         self.__cbs = self.__parse_cbscores(json.get("cbs"))
 
@@ -61,8 +69,8 @@ class RemoteVisitorData:
         data_to_add: List[BaseData] = []
         if self.__custom_data_dict:
             data_to_add += self.__custom_data_dict.values()
-        if self.__previous_visitor_visits:
-            data_to_add.append(self.__previous_visitor_visits)
+        if self.__visitor_visits:
+            data_to_add.append(self.__visitor_visits)
         if self.__kcs_heat:
             data_to_add.append(self.__kcs_heat)
         if self.__cbs:
@@ -71,6 +79,8 @@ class RemoteVisitorData:
             data_to_add += self.__page_view_visits.values()
         if self.__experiments:
             data_to_add += self.__experiments.values()
+        if self.__personalizations:
+            data_to_add += self.__personalizations.values()
         if self.__conversions:
             data_to_add += self.__conversions
         data_to_add += self.__single_data()
@@ -113,7 +123,7 @@ class RemoteVisitorData:
         """Returns the `visitorCode` from the most recent visit"""
         return self.__visitor_code
 
-    def __parse_visit(self, json_visit: Dict[str, Any]) -> None:
+    def __parse_visit(self, json_visit: Dict[str, Any], is_prev_visit: bool) -> None:
         if (self.__visitor_code is None) and isinstance(visitor_code := json_visit.get("visitorCode"), str):
             self.__visitor_code = visitor_code
         custom_data_events = json_visit.get("customDataEvents")
@@ -133,7 +143,10 @@ class RemoteVisitorData:
             self.__parse_geolocation(geolocation_events[-1])
         static_data_event = json_visit.get("staticDataEvent")
         if static_data_event:
-            self.__parse_static_data(static_data_event)
+            self.__parse_static_data(static_data_event, is_prev_visit)
+        personalization_events = json_visit.get("personalizationEvents")
+        if personalization_events:
+            self.__parse_personalizations(personalization_events)
 
     def __parse_custom_data(self, custom_data_events: List[Dict[str, Any]]) -> None:
         if self.__custom_data_dict is None:
@@ -196,35 +209,49 @@ class RemoteVisitorData:
             if country is not None:
                 self.__geolocation = Geolocation(country, data.get("region"), data.get("city"))
 
-    def __parse_static_data(self, static_data_event: Dict[str, Any]) -> None:
+    def __parse_static_data(self, static_data_event: Dict[str, Any], is_prev_visit: bool) -> None:
         if self.__device and self.__browser and self.__operating_system:
             return
         data = static_data_event.get("data")
         if data is not None:
-            if self.__device is None:
+            if self.__visit_number == 0:
+                self.__visit_number = data.get("visitNumber", 0)
+                if is_prev_visit and (self.__visit_number > 0):
+                    self.__visit_number += 1
+            if self.__filter.device and (self.__device is None):
                 device_type = enum_from_name_literal(data.get("deviceType"), DeviceType, None)
                 if device_type is not None:
                     self.__device = Device(device_type)
-            if self.__browser is None:
+            if self.__filter.browser and (self.__browser is None):
                 browser_type = enum_from_name_literal(data.get("browser"), BrowserType, None)
                 if browser_type is not None:
                     self.__browser = Browser(browser_type, data.get("browserVersion"))
-            if self.__operating_system is None:
+            if self.__filter.operating_system and (self.__operating_system is None):
                 operating_system_type = enum_from_name_literal(data.get("os"), OperatingSystemType, None)
                 if operating_system_type is not None:
                     self.__operating_system = OperatingSystem(operating_system_type)
 
-    def __single_data(self) -> List[Data]:
-        objects: List[Data] = []
+    def __parse_personalizations(self, personalization_events: List[Dict[str, Any]]) -> None:
+        if self.__personalizations is None:
+            self.__personalizations = {}
+        for personalization_event in reversed(personalization_events):
+            data = personalization_event.get("data")
+            if data is not None:
+                index = data.get("id")
+                variation_id = data.get("variationId")
+                if index is not None and variation_id is not None:
+                    if index not in self.__personalizations:
+                        self.__personalizations[index] = Personalization(index, variation_id)
+
+    def __single_data(self) -> Iterable[Data]:
         if self.__device:
-            objects.append(self.__device)
+            yield self.__device
         if self.__browser:
-            objects.append(self.__browser)
+            yield self.__browser
         if self.__operating_system:
-            objects.append(self.__operating_system)
+            yield self.__operating_system
         if self.__geolocation:
-            objects.append(self.__geolocation)
-        return objects
+            yield self.__geolocation
 
     @staticmethod
     def __parse_kcs_heat(kcs: Optional[Any]) -> Optional[KcsHeat]:
