@@ -1,4 +1,5 @@
 """Network"""
+
 from typing import Optional
 from kameleoon.network.access_token_source_factory import AccessTokenSourceFactory
 from kameleoon.network.url_provider import UrlProvider
@@ -16,18 +17,20 @@ from kameleoon.network.services.data_service_impl import DataServiceImpl
 
 class NetworkManagerFactoryImpl(NetworkManagerFactory):
     """Network manager factory implementation"""
+
     # fmt is disabled due issue in pylint, disabling R0801 doesn’t work
     # fmt: off
     # pylint: disable=R0913
     def create(
         self,
         site_code: str,
-        environment: Optional[str], call_timeout: float,
+        environment: Optional[str], call_timeout: float,  # don`t change this line
         net_provider: NetProvider,
         access_token_source_factory: AccessTokenSourceFactory,
+        network_domain: Optional[str],
     ) -> NetworkManager:
         # fmt: on
-        url_provider = UrlProvider(site_code, UrlProvider.DEFAULT_DATA_API_DOMAIN)
+        url_provider = UrlProvider(site_code, network_domain)
         network_manager = NetworkManagerImpl(
             url_provider, environment, call_timeout, net_provider, access_token_source_factory,
         )
