@@ -1,7 +1,7 @@
 """Time elapsed since visit condition"""
 
 import time
-from typing import Any, Dict, Union, Optional
+from typing import Any, Dict, Union
 from kameleoon.data.visitor_visits import VisitorVisits
 from kameleoon.targeting.conditions.targeting_condition import TargetingConditionType
 from kameleoon.targeting.conditions.number_condition import NumberCondition
@@ -14,14 +14,14 @@ class TimeElapsedSinceVisitCondition(NumberCondition):
         super().__init__(json_condition, "countInMillis")
         self.__is_first_visit = self.type == TargetingConditionType.FIRST_VISIT.name
 
-    def check(self, data: Optional[VisitorVisits]) -> bool:
-        return self._condition_value is not None and self.__check(data)
+    # pylint: disable=C0103
+    def check(self, data: Any) -> bool:
+        visitor_visits, ok = VisitorVisits.get_visitor_visits(data)
+        return ok and (self._condition_value is not None) and self.__check(visitor_visits)
 
-    def __check(self, visitor_visits: Optional[VisitorVisits]) -> bool:
-        timestamps = VisitorVisits.get_previous_visit_timestamps(visitor_visits)
-        if len(timestamps) < 1:
+    def __check(self, visitor_visits: VisitorVisits) -> bool:
+        if len(visitor_visits.prev_visits) == 0:
             return False
-
         now = int(time.time() * 1000)  # Convert seconds to milliseconds
-        visit_time = timestamps[-1 if self.__is_first_visit else 0]
-        return self._check_targeting(now - visit_time)
+        visit = visitor_visits.prev_visits[-1 if self.__is_first_visit else 0]
+        return self._check_targeting(now - visit.time_started)
